Package: hydroGOF
Type: Package
Title: Goodness-of-Fit Functions for Comparison of Simulated and
        Observed Hydrological Time Series
Version: 0.7-0
Authors@R: c( person("Mauricio Zambrano-Bigiarini", email = "mzb.devel@gmail.com", role=c("aut", "cre", "cph"), comment=c(ORCID = "0000-0002-9536-643X")) )
Maintainer: Mauricio Zambrano-Bigiarini <mzb.devel@gmail.com>
Description: S3 functions implementing both statistical and graphical goodness-of-fit measures between observed and simulated values, mainly oriented to be used during the calibration, validation, and application of hydrological models. Missing values in observed and/or simulated values can be removed before computations. Comments / questions / collaboration of any kind are very welcomed.
License: GPL (>= 2)
Depends: R (>= 2.10.0), zoo (>= 1.7-2)
Imports: hydroTSM (>= 0.8-6), xts (>= 0.8-2), methods, stats
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
URL: https://hzambran.github.io/hydroGOF/,
        https://github.com/hzambran/hydroGOF,
        https://CRAN.R-project.org/package=hydroGOF
MailingList: https://stat.ethz.ch/mailman/listinfo/r-sig-ecology
BugReports: https://github.com/hzambran/hydroGOF/issues
LazyLoad: yes
NeedsCompilation: no
Repository: CRAN
Packaged: 2026-04-30 19:52:31 UTC; hzambran
Author: Mauricio Zambrano-Bigiarini [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0002-9536-643X>)
Date/Publication: 2026-05-01 05:10:38 UTC
Built: R 4.6.0; ; 2026-05-01 21:29:12 UTC; unix
