import(methods)
importFrom("grDevices", "xy.coords")
importFrom("graphics", "Axis", "abline", "axis", "barplot", "box", "grid", "layout", "legend", "lines", "par", "plot", "plot.new", "points", "polygon", "hist")
importFrom("stats", "coefficients", "cor", "lm", "sd", "time", "window", "na.omit", "median", "quantile", "start", "end", "density")
importFrom("xts", .index, periodicity, axTicksByTime)
importFrom("zoo", is.zoo, as.zoo, coredata, plot.zoo, "time<-", "merge.zoo", "MATCH")
importFrom("hydroTSM", sfreq, vector2zoo, daily2monthly, daily2annual, monthly2annual, dm2seasonal, drawTimeAxis, time2season, seasonalfunction, fdc, fdc.default, fdc.zoo, fdc.matrix, fdc.data.frame, shiftyears)

exportPattern("^[^\\.]")

S3method(KGE, data.frame) 
S3method(KGE, default) 
S3method(KGE, matrix) 
S3method(KGE, zoo) 
S3method(KGElf, data.frame) 
S3method(KGElf, default) 
S3method(KGElf, matrix) 
S3method(KGElf, zoo) 
S3method(sKGE, data.frame) 
S3method(sKGE, default) 
S3method(sKGE, matrix) 
S3method(sKGE, zoo) 
S3method(KGEnp, data.frame) 
S3method(KGEnp, default) 
S3method(KGEnp, matrix) 
S3method(KGEnp, zoo) 
S3method(KGEkm, data.frame) 
S3method(KGEkm, default) 
S3method(KGEkm, matrix) 
S3method(KGEkm, zoo) 
S3method(NSE, data.frame)
S3method(NSE, default) 
S3method(NSE, matrix) 
S3method(NSE, zoo) 
S3method(VE, data.frame)
S3method(VE, default) 
S3method(VE, matrix) 
S3method(VE, zoo) 
S3method(APFB, data.frame) 
S3method(APFB, default) 
S3method(APFB, matrix) 
S3method(APFB, zoo)
S3method(HFB, data.frame) 
S3method(HFB, default) 
S3method(HFB, matrix) 
S3method(HFB, zoo)  
S3method(br2, data.frame) 
S3method(br2, default) 
S3method(br2, matrix)
S3method(br2, zoo) 
S3method(cp, data.frame) 
S3method(cp, default) 
S3method(cp, matrix) 
S3method(cp, zoo) 
S3method(d, data.frame)
S3method(d, default) 
S3method(d, matrix) 
S3method(d, zoo) 
S3method(dr, data.frame)
S3method(dr, default) 
S3method(dr, matrix) 
S3method(dr, zoo) 
S3method(gof, data.frame) 
S3method(gof, default) 
S3method(gof, matrix) 
S3method(gof, zoo)
S3method(mNSE, data.frame) 
S3method(mNSE, default) 
S3method(mNSE, matrix) 
S3method(mNSE, zoo) 
S3method(mae, data.frame)
S3method(mae, default) 
S3method(mae, matrix) 
S3method(mae, zoo) 
S3method(md, data.frame) 
S3method(md, default) 
S3method(md, matrix)
S3method(md, zoo) 
S3method(me, data.frame) 
S3method(me, default) 
S3method(me, matrix) 
S3method(me, zoo) 
S3method(mse, data.frame)
S3method(mse, default) 
S3method(mse, matrix) 
S3method(mse, zoo) 
S3method(nrmse, data.frame) 
S3method(nrmse, default)
S3method(nrmse, matrix) 
S3method(nrmse, zoo) 
S3method(pbias, data.frame) 
S3method(pbias, default) 
S3method(pbias, matrix)
S3method(pbias, zoo) 
S3method(pbiasfdc, data.frame) 
S3method(pbiasfdc, default) 
S3method(pbiasfdc, matrix)
S3method(pbiasfdc, zoo) 
S3method(pfactor, data.frame) 
S3method(pfactor, default)
S3method(pfactor, matrix)
S3method(rNSE, data.frame) 
S3method(rNSE, default) 
S3method(rNSE, matrix) 
S3method(rNSE, zoo) 
S3method(R2, data.frame)
S3method(R2, default) 
S3method(R2, matrix)
S3method(R2, zoo) 
S3method(rPearson, data.frame)
S3method(rPearson, default) 
S3method(rPearson, matrix)
S3method(rPearson, zoo) 
S3method(rSD, data.frame)
S3method(rSD, default) 
S3method(rSD, matrix) 
S3method(rSD, zoo) 
S3method(rSpearman, data.frame)
S3method(rSpearman, default) 
S3method(rSpearman, matrix)
S3method(rSpearman, zoo) 
S3method(rd, data.frame) 
S3method(rd, default) 
S3method(rd, matrix)
S3method(rd, zoo) 
S3method(rfactor, data.frame) 
S3method(rfactor, default) 
S3method(rfactor, matrix)
S3method(rmse, data.frame) 
S3method(rmse, default) 
S3method(rmse, matrix) 
S3method(rmse, zoo) 
S3method(rsr, data.frame)
S3method(rsr, default) 
S3method(rsr, matrix) 
S3method(rsr, zoo) 
S3method(ssq, data.frame) 
S3method(ssq, default) 
S3method(ssq, matrix)
S3method(ssq, zoo)
S3method(ubRMSE, data.frame) 
S3method(ubRMSE, default)
S3method(ubRMSE, matrix) 
S3method(ubRMSE, zoo) 
S3method(valindex, default) 
S3method(valindex, matrix)
S3method(wsNSE, data.frame) 
S3method(wsNSE, default) 
S3method(wsNSE, matrix) 
S3method(wsNSE, zoo) 
S3method(wNSE, data.frame) 
S3method(wNSE, default) 
S3method(wNSE, matrix) 
S3method(wNSE, zoo) 
S3method(PMR, data.frame)
S3method(PMR, default) 
S3method(PMR, matrix) 
S3method(PMR, zoo) 
S3method(JDKGE, data.frame)
S3method(JDKGE, default) 
S3method(JDKGE, matrix) 
S3method(JDKGE, zoo) 
S3method(LME, data.frame)
S3method(LME, default) 
S3method(LME, matrix) 
S3method(LME, zoo) 
S3method(LCE, data.frame)
S3method(LCE, default) 
S3method(LCE, matrix) 
S3method(LCE, zoo) 
